@@ -55,10 +55,12 @@ static struct ovn_mirror *ovn_mirror_find(struct shash *ovn_mirrors,
 static void ovn_mirror_delete(struct ovn_mirror *);
 static void ovn_mirror_add_lport(struct ovn_mirror *, struct local_binding *);
 static void sync_ovn_mirror(struct ovn_mirror *, struct ovsdb_idl_txn *,
-                            const struct ovsrec_bridge *);
+                            const struct ovsrec_bridge *,
+                            struct shash *ovs_mirror_ports);
 
 static void create_ovs_mirror(struct ovn_mirror *, struct ovsdb_idl_txn *,
-                              const struct ovsrec_bridge *);
+                              const struct ovsrec_bridge *,
+                              struct shash *ovs_mirror_ports);
 static struct ovsrec_port *create_mirror_port(struct ovn_mirror *,
                                               struct ovsdb_idl_txn *,
                                               const struct ovsrec_bridge *);
@@ -75,6 +77,9 @@ static const struct ovsrec_port *get_iface_port(
 
 char *get_mirror_tunnel_type(const struct sbrec_mirror *);
 
+static void build_ovs_mirror_ports(const struct ovsrec_bridge *,
+                                   struct shash *ovs_mirror_ports);
+
 void
 mirror_register_ovs_idl(struct ovsdb_idl *ovs_idl)
 {
@@ -110,7 +115,8 @@ mirror_run(struct ovsdb_idl_txn *ovs_idl_txn,
     }
 
     struct shash ovn_mirrors = SHASH_INITIALIZER(&ovn_mirrors);
-    struct shash tmp_mirrors = SHASH_INITIALIZER(&tmp_mirrors);
+    struct shash ovs_local_mirror_ports =
+        SHASH_INITIALIZER(&ovs_local_mirror_ports);
 
     /* Iterate through sb mirrors and build the 'ovn_mirrors'. */
     const struct sbrec_mirror *sb_mirror;
@@ -142,6 +148,8 @@ mirror_run(struct ovsdb_idl_txn *ovs_idl_txn,
         return;
     }
 
+    build_ovs_mirror_ports(br_int, &ovs_local_mirror_ports);
+
     /* Iterate through the local bindings and if the local binding's 'pb' has
      * mirrors associated, add it to the ovn_mirror. */
     struct shash_node *node;
@@ -166,7 +174,7 @@ mirror_run(struct ovsdb_idl_txn *ovs_idl_txn,
      * create/update or delete the ovsrec mirror(s). */
      SHASH_FOR_EACH (node, &ovn_mirrors) {
         struct ovn_mirror *m = node->data;
-        sync_ovn_mirror(m, ovs_idl_txn, br_int);
+        sync_ovn_mirror(m, ovs_idl_txn, br_int, &ovs_local_mirror_ports);
      }
 
     SHASH_FOR_EACH_SAFE (node, &ovn_mirrors) {
@@ -175,9 +183,52 @@ mirror_run(struct ovsdb_idl_txn *ovs_idl_txn,
     }
 
     shash_destroy(&ovn_mirrors);
+    shash_destroy(&ovs_local_mirror_ports);
 }
 
 /* Static functions. */
+
+/* Builds mapping from mirror-id to ovsrec_port.
+ */
+static void
+build_ovs_mirror_ports(const struct ovsrec_bridge *br_int,
+                       struct shash *ovs_mirror_ports)
+{
+    int i;
+    for (i = 0; i < br_int->n_ports; i++) {
+        const struct ovsrec_port *port_rec = br_int->ports[i];
+        int j;
+
+        if (!strcmp(port_rec->name, br_int->name)) {
+            continue;
+        }
+
+        for (j = 0; j < port_rec->n_interfaces; j++) {
+            const struct ovsrec_interface *iface_rec;
+
+            iface_rec = port_rec->interfaces[j];
+            const char *mirror_id = smap_get(&iface_rec->external_ids,
+                                             "mirror-id");
+            if (mirror_id) {
+                const struct ovsrec_port *p = shash_find_data(ovs_mirror_ports,
+                                                              mirror_id);
+                if (!p) {
+                    shash_add(ovs_mirror_ports, mirror_id, port_rec);
+                } else {
+                    static struct vlog_rate_limit rl =
+                        VLOG_RATE_LIMIT_INIT(1, 5);
+                    VLOG_WARN_RL(
+                        &rl,
+                        "Invalid configuration: same mirror-id [%s] is "
+                        "configured on interfaces of ports: [%s] and [%s]. "
+                        "Ignoring the configuration on interface [%s]",
+                        mirror_id, port_rec->name, p->name, iface_rec->name);
+                }
+            }
+        }
+    }
+}
+
 static struct ovn_mirror *
 ovn_mirror_create(char *mirror_name)
 {
@@ -219,11 +270,17 @@ ovn_mirror_delete(struct ovn_mirror *m)
 static void
 ovn_mirror_add_lport(struct ovn_mirror *m, struct local_binding *lbinding)
 {
-    struct mirror_lport *m_lport = xzalloc(sizeof *m_lport);
-    m_lport->lbinding = lbinding;
-    if (!strcmp(m->sb_mirror->filter, "from-lport")) {
+    if (!strcmp(m->sb_mirror->filter, "from-lport") ||
+        !strcmp(m->sb_mirror->filter, "both")) {
+        struct mirror_lport *m_lport = xzalloc(sizeof *m_lport);
+        m_lport->lbinding = lbinding;
         ovs_list_push_back(&m->mirror_src_lports, &m_lport->list_node);
-    } else {
+    }
+
+    if (!strcmp(m->sb_mirror->filter, "to-lport") ||
+        !strcmp(m->sb_mirror->filter, "both")) {
+        struct mirror_lport *m_lport = xzalloc(sizeof *m_lport);
+        m_lport->lbinding = lbinding;
         ovs_list_push_back(&m->mirror_dst_lports, &m_lport->list_node);
     }
 }
@@ -280,7 +337,8 @@ check_and_update_interface_table(struct ovn_mirror *m,
 
 static void
 sync_ovn_mirror(struct ovn_mirror *m, struct ovsdb_idl_txn *ovs_idl_txn,
-                const struct ovsrec_bridge *br_int)
+                const struct ovsrec_bridge *br_int,
+                struct shash *ovs_mirror_ports)
 {
     if (should_delete_ovs_mirror(m)) {
         /* Delete the ovsrec mirror. */
@@ -295,9 +353,31 @@ sync_ovn_mirror(struct ovn_mirror *m, struct ovsdb_idl_txn *ovs_idl_txn,
     }
 
     if (m->sb_mirror && !m->ovs_mirror) {
-        create_ovs_mirror(m, ovs_idl_txn, br_int);
-    } else {
+        create_ovs_mirror(m, ovs_idl_txn, br_int, ovs_mirror_ports);
+        if (!m->ovs_mirror) {
+            return;
+        }
+    } else if (strcmp(m->sb_mirror->type, "local")) {
         check_and_update_interface_table(m, ovs_idl_txn, br_int);
+
+        /* For upgradability, set the "ovn-owned" in case it was not set when
+         * the port was created. */
+        if (!smap_get_bool(&m->ovs_mirror->output_port->external_ids,
+                                   "ovn-owned", false)) {
+            ovsrec_port_update_external_ids_setkey(m->ovs_mirror->output_port,
+                                                   "ovn-owned", "true");
+        }
+    } else {
+        /* type is local, check mirror-id */
+        const struct ovsrec_port *mirror_port =
+            shash_find_data(ovs_mirror_ports, m->sb_mirror->sink);
+        if (!mirror_port) {
+            delete_ovs_mirror(m, br_int);
+            return;
+        }
+        if (mirror_port != m->ovs_mirror->output_port) {
+            ovsrec_mirror_set_output_port(m->ovs_mirror, mirror_port);
+        }
     }
 
     sync_ovs_mirror_ports(m, br_int);
@@ -352,19 +432,32 @@ create_mirror_port(struct ovn_mirror *m, struct ovsdb_idl_txn *ovs_idl_txn,
     ovsrec_port_set_interfaces(port, &iface, 1);
     ovsrec_bridge_update_ports_addvalue(br_int, port);
 
+    const struct smap port_external_ids =
+        SMAP_CONST1(&port_external_ids, "ovn-owned", "true");
+    ovsrec_port_set_external_ids(port, &port_external_ids);
+
     free(port_name);
     return port;
 }
 
 static void
 create_ovs_mirror(struct ovn_mirror *m, struct ovsdb_idl_txn *ovs_idl_txn,
-                  const struct ovsrec_bridge *br_int)
+                  const struct ovsrec_bridge *br_int,
+                  struct shash *ovs_mirror_ports)
 {
-    const struct ovsrec_port *port = create_mirror_port(m, ovs_idl_txn,
-                                                        br_int);
+    const struct ovsrec_port *mirror_port;
+    if (!strcmp(m->sb_mirror->type, "local")) {
+        mirror_port = shash_find_data(ovs_mirror_ports, m->sb_mirror->sink);
+        if (!mirror_port) {
+            return;
+        }
+    } else {
+        mirror_port = create_mirror_port(m, ovs_idl_txn, br_int);
+    }
+
     m->ovs_mirror = ovsrec_mirror_insert(ovs_idl_txn);
     ovsrec_mirror_set_name(m->ovs_mirror, m->name);
-    ovsrec_mirror_set_output_port(m->ovs_mirror, port);
+    ovsrec_mirror_set_output_port(m->ovs_mirror, mirror_port);
 
     const struct smap external_ids =
         SMAP_CONST1(&external_ids, "ovn-owned", "true");
@@ -418,11 +511,16 @@ sync_ovs_mirror_ports(struct ovn_mirror *m, const struct ovsrec_bridge *br_int)
 static void
 delete_ovs_mirror(struct ovn_mirror *m, const struct ovsrec_bridge *br_int)
 {
-    if (m->ovs_mirror->output_port) {
-        ovsrec_bridge_update_ports_delvalue(br_int,
-                                            m->ovs_mirror->output_port);
-        ovsrec_port_delete(m->ovs_mirror->output_port);
-    }
     ovsrec_bridge_update_mirrors_delvalue(br_int, m->ovs_mirror);
+    if (m->ovs_mirror->output_port) {
+        bool ovn_owned =
+            smap_get_bool(&m->ovs_mirror->output_port->external_ids,
+                          "ovn-owned", false);
+        if (ovn_owned) {
+            ovsrec_bridge_update_ports_delvalue(br_int,
+                                                m->ovs_mirror->output_port);
+            ovsrec_port_delete(m->ovs_mirror->output_port);
+        }
+    }
     ovsrec_mirror_delete(m->ovs_mirror);
 }
