@@ -142,15 +142,9 @@ destroy_port_ctx(struct vif_plug_port_ctx *ctx)
 {
     smap_destroy(&ctx->vif_plug_port_ctx_in.lport_options);
     smap_destroy(&ctx->vif_plug_port_ctx_in.iface_options);
-    if (ctx->vif_plug_port_ctx_in.lport_name) {
-        free((char *)ctx->vif_plug_port_ctx_in.lport_name);
-    }
-    if (ctx->vif_plug_port_ctx_in.iface_name) {
-        free((char *)ctx->vif_plug_port_ctx_in.iface_name);
-    }
-    if (ctx->vif_plug_port_ctx_in.iface_type) {
-        free((char *)ctx->vif_plug_port_ctx_in.iface_type);
-    }
+    free(CONST_CAST(char *, ctx->vif_plug_port_ctx_in.lport_name));
+    free(CONST_CAST(char *, ctx->vif_plug_port_ctx_in.iface_name));
+    free(CONST_CAST(char *, ctx->vif_plug_port_ctx_in.iface_type));
     /* Note that data associated with ctx->vif_plug_port_ctx_out must be
      * destroyed by the plug provider implementation with a call to
      * vif_plug_port_ctx_destroy prior to calling this function */
@@ -532,22 +526,14 @@ vif_plug_handle_iface(const struct ovsrec_interface *iface_rec,
  * completeness of the initial data downloading we need this counter so that we
  * do not erronously unplug ports because the data is just not loaded yet.
  */
-#define VIF_PLUG_PRIME_IDL_COUNT_SEEED 10
-static int vif_plug_prime_idl_count = VIF_PLUG_PRIME_IDL_COUNT_SEEED;
-
-void
-vif_plug_reset_idl_prime_counter(void)
-{
-    vif_plug_prime_idl_count = VIF_PLUG_PRIME_IDL_COUNT_SEEED;
-}
-
 void
 vif_plug_run(struct vif_plug_ctx_in *vif_plug_ctx_in,
              struct vif_plug_ctx_out *vif_plug_ctx_out)
 {
-    if (vif_plug_prime_idl_count && --vif_plug_prime_idl_count > 0) {
-        VLOG_DBG("vif_plug_run: vif_plug_prime_idl_count=%d, will not unplug "
-                 "ports in this iteration.", vif_plug_prime_idl_count);
+    bool delay_plug = daemon_started_recently();
+    if (delay_plug) {
+        VLOG_DBG("vif_plug_run: daemon started recently, will not unplug "
+                 "ports in this iteration.");
     }
 
     if (!vif_plug_ctx_in->chassis_rec) {
@@ -557,7 +543,7 @@ vif_plug_run(struct vif_plug_ctx_in *vif_plug_ctx_in,
     OVSREC_INTERFACE_TABLE_FOR_EACH (iface_rec,
                                      vif_plug_ctx_in->iface_table) {
         vif_plug_handle_iface(iface_rec, vif_plug_ctx_in, vif_plug_ctx_out,
-                              !vif_plug_prime_idl_count);
+                              !delay_plug);
     }
 
     struct sbrec_port_binding *target =
@@ -573,7 +559,7 @@ vif_plug_run(struct vif_plug_ctx_in *vif_plug_ctx_in,
         enum en_lport_type lport_type = get_lport_type(pb);
         if (lport_type == LP_VIF) {
             vif_plug_handle_lport_vif(pb, vif_plug_ctx_in, vif_plug_ctx_out,
-                                      !vif_plug_prime_idl_count);
+                                      !delay_plug);
         }
     }
     sbrec_port_binding_index_destroy_row(target);
@@ -582,8 +568,8 @@ vif_plug_run(struct vif_plug_ctx_in *vif_plug_ctx_in,
 static void
 vif_plug_finish_deleted__(struct shash *deleted_iface_ids, bool txn_success)
 {
-    struct shash_node *node, *next;
-    SHASH_FOR_EACH_SAFE (node, next, deleted_iface_ids) {
+    struct shash_node *node;
+    SHASH_FOR_EACH_SAFE (node, deleted_iface_ids) {
         struct vif_plug_port_ctx *vif_plug_port_ctx = node->data;
         if (txn_success) {
             vif_plug_port_finish(vif_plug_port_ctx->vif_plug,
@@ -608,8 +594,8 @@ vif_plug_finish_deleted(struct shash *deleted_iface_ids) {
 static void
 vif_plug_finish_changed__(struct shash *changed_iface_ids, bool txn_success)
 {
-    struct shash_node *node, *next;
-    SHASH_FOR_EACH_SAFE (node, next, changed_iface_ids) {
+    struct shash_node *node;
+    SHASH_FOR_EACH_SAFE (node, changed_iface_ids) {
         struct vif_plug_port_ctx *vif_plug_port_ctx = node->data;
         if (txn_success) {
             vif_plug_port_finish(vif_plug_port_ctx->vif_plug,
